// Intersector.cpp
//

// LDraw intersection calculator
// Calculates intersection between a "cut" file and a "cutter" file. All elements of cut file are
// split where they meet cutter file surface. The intersection line is to be created using Isecalc.

// Usage: Intersector [-c] [-t] [-s <scale>] LdrawCutFile LdrawCutterFile LdrawCutFileOut

// Primitives must be inlined before usage if needed.
// When -c is used, coloring of cut elements is done according to orientation of cutter surface.
// No triangle condensing is done when -t option is used.

// Philippe "Philo" Hurbain - 2008 - www.philohome.com

// Contains triangle intersection code from Tomas Mller
// See thesis here: http://www.cs.lth.se/home/Tomas_Akenine_Moller/pubs/tritri.pdf
// Complete code here: http://jgt.akpeters.com/papers/Moller97/tritri.html


// Contains softSurfer Line - Triangle intersection (adaptation)
// Contains softSurfer Point - Line distance calculation (adaptation)
// Copyright 2001, softSurfer (www.softsurfer.com)
// This code may be freely used and modified for any purpose
// providing that this copyright notice is included with it.
// SoftSurfer makes no warranty for this code, and cannot be held
// liable for any real or imagined damage resulting from its use.
// Users of this code must verify correctness for their application.

#include "stdafx.h"

/* Triangle/triangle intersection test routine,
 * by Tomas Moller, 1997.
 * See article "A Fast Triangle-Triangle Intersection Test",
 * Journal of Graphics Tools, 2(2), 1997
 * updated: 2001-06-20 (added line of intersection)
 *
 * This version computes the line of intersection as well (if they are not coplanar):
 * int tri_tri_intersect_with_isectline(double V0[3],double V1[3],double V2[3], 
 *				        double U0[3],double U1[3],double U2[3],int *coplanar,
 *				        double isectpt1[3],double isectpt2[3]);
 * coplanar returns whether the tris are coplanar
 * isectpt1, isectpt2 are the endpoints of the line of intersection
 */

#include <math.h>
#include <stdlib.h>
#include <time.h>


#define USE_EPSILON_TEST TRUE  
#define EPSILON 0.00001

/* Null vector */

double null[3] = {0,0,0};

/* some macros */

#define CROSS(dest,v1,v2)                      \
              dest[0]=v1[1]*v2[2]-v1[2]*v2[1]; \
              dest[1]=v1[2]*v2[0]-v1[0]*v2[2]; \
              dest[2]=v1[0]*v2[1]-v1[1]*v2[0];

#define DOT(v1,v2) (v1[0]*v2[0]+v1[1]*v2[1]+v1[2]*v2[2])

#define SUB(dest,v1,v2) dest[0]=v1[0]-v2[0]; dest[1]=v1[1]-v2[1]; dest[2]=v1[2]-v2[2]; 

#define ADD(dest,v1,v2) dest[0]=v1[0]+v2[0]; dest[1]=v1[1]+v2[1]; dest[2]=v1[2]+v2[2]; 

#define MULT(dest,v,factor) dest[0]=factor*v[0]; dest[1]=factor*v[1]; dest[2]=factor*v[2];

#define SET(dest,src) dest[0]=src[0]; dest[1]=src[1]; dest[2]=src[2];

#define MANHATTAN(v1, v2) (fabs(v1[0]-v2[0]) + fabs(v1[1]-v2[1]) + fabs(v1[2]-v2[2]))

#define NORM(v1) (sqrt(DOT(v1,v1)))

#define DIST(p1, p2) (sqrt((p1[0]-p2[0])*(p1[0]-p2[0])+(p1[1]-p2[1])*(p1[1]-p2[1])+(p1[2]-p2[2])*(p1[2]-p2[2])))


#define ROTINV(p1,p2,p3) { double temp[3]; SET(temp,p1); SET(p1,p2); SET(p2,p3); SET(p3,temp); }

#define ROTDIR(p1,p2,p3) { double temp[3]; SET(temp,p1); SET(p1,p3); SET(p3,p2); SET(p2,temp); }

#define ROT4(p1,p2,p3, p4) { double temp[3]; SET(temp,p1); SET(p1,p4); SET(p4,p3); SET(p3,p2); SET(p2,temp); }

#define SWAP(p1,p2) { double temp[3]; SET(temp,p1); SET(p1,p2); SET(p2,temp); }


/* sort so that a<=b */
#define SORT(a,b)       \
             if(a>b)    \
             {          \
               double c; \
               c=a;     \
               a=b;     \
               b=c;     \
             }

#define ISECT(VV0,VV1,VV2,D0,D1,D2,isect0,isect1) \
              isect0=VV0+(VV1-VV0)*D0/(D0-D1);    \
              isect1=VV0+(VV2-VV0)*D0/(D0-D2);


#define COMPUTE_INTERVALS(VV0,VV1,VV2,D0,D1,D2,D0D1,D0D2,isect0,isect1) \
  if(D0D1>0.0f)                                         \
  {                                                     \
    /* here we know that D0D2<=0.0 */                   \
    /* that is D0, D1 are on the same side, D2 on the other or on the plane */ \
    ISECT(VV2,VV0,VV1,D2,D0,D1,isect0,isect1);          \
  }                                                     \
  else if(D0D2>0.0f)                                    \
  {                                                     \
    /* here we know that d0d1<=0.0 */                   \
    ISECT(VV1,VV0,VV2,D1,D0,D2,isect0,isect1);          \
  }                                                     \
  else if(D1*D2>0.0f || D0!=0.0f)                       \
  {                                                     \
    /* here we know that d0d1<=0.0 or that D0!=0.0 */   \
    ISECT(VV0,VV1,VV2,D0,D1,D2,isect0,isect1);          \
  }                                                     \
  else if(D1!=0.0f)                                     \
  {                                                     \
    ISECT(VV1,VV0,VV2,D1,D0,D2,isect0,isect1);          \
  }                                                     \
  else if(D2!=0.0f)                                     \
  {                                                     \
    ISECT(VV2,VV0,VV1,D2,D0,D1,isect0,isect1);          \
  }                                                     \
  else                                                  \
  {                                                     \
    /* triangles are coplanar */                        \
    return coplanar_tri_tri(N1,V0,V1,V2,U0,U1,U2);      \
  }



/* this edge to edge test is based on Franlin Antonio's gem:
   "Faster Line Segment Intersection", in Graphics Gems III,
   pp. 199-202 */ 
#define EDGE_EDGE_TEST(V0,U0,U1)                      \
  Bx=U0[i0]-U1[i0];                                   \
  By=U0[i1]-U1[i1];                                   \
  Cx=V0[i0]-U0[i0];                                   \
  Cy=V0[i1]-U0[i1];                                   \
  f=Ay*Bx-Ax*By;                                      \
  d=By*Cx-Bx*Cy;                                      \
  if((f>0 && d>=0 && d<=f) || (f<0 && d<=0 && d>=f))  \
  {                                                   \
    e=Ax*Cy-Ay*Cx;                                    \
    if(f>0)                                           \
    {                                                 \
      if(e>=0 && e<=f) return 1;                      \
    }                                                 \
    else                                              \
    {                                                 \
      if(e<=0 && e>=f) return 1;                      \
    }                                                 \
  }                                

#define EDGE_AGAINST_TRI_EDGES(V0,V1,U0,U1,U2) \
{                                              \
  double Ax,Ay,Bx,By,Cx,Cy,e,d,f;               \
  Ax=V1[i0]-V0[i0];                            \
  Ay=V1[i1]-V0[i1];                            \
  /* test edge U0,U1 against V0,V1 */          \
  EDGE_EDGE_TEST(V0,U0,U1);                    \
  /* test edge U1,U2 against V0,V1 */          \
  EDGE_EDGE_TEST(V0,U1,U2);                    \
  /* test edge U2,U1 against V0,V1 */          \
  EDGE_EDGE_TEST(V0,U2,U0);                    \
}

#define POINT_IN_TRI(V0,U0,U1,U2)           \
{                                           \
  double a,b,c,d0,d1,d2;                     \
  /* is T1 completly inside T2? */          \
  /* check if V0 is inside tri(U0,U1,U2) */ \
  a=U1[i1]-U0[i1];                          \
  b=-(U1[i0]-U0[i0]);                       \
  c=-a*U0[i0]-b*U0[i1];                     \
  d0=a*V0[i0]+b*V0[i1]+c;                   \
                                            \
  a=U2[i1]-U1[i1];                          \
  b=-(U2[i0]-U1[i0]);                       \
  c=-a*U1[i0]-b*U1[i1];                     \
  d1=a*V0[i0]+b*V0[i1]+c;                   \
                                            \
  a=U0[i1]-U2[i1];                          \
  b=-(U0[i0]-U2[i0]);                       \
  c=-a*U2[i0]-b*U2[i1];                     \
  d2=a*V0[i0]+b*V0[i1]+c;                   \
  if(d0*d1>0.0)                             \
  {                                         \
    if(d0*d2>0.0) return 1;                 \
  }                                         \
}

int coplanar_tri_tri(double N[3],double V0[3],double V1[3],double V2[3],
                     double U0[3],double U1[3],double U2[3])
{
   double A[3];
   short i0,i1;
   /* first project onto an axis-aligned plane, that maximizes the area */
   /* of the triangles, compute indices: i0,i1. */
   A[0]=fabs(N[0]);
   A[1]=fabs(N[1]);
   A[2]=fabs(N[2]);
   if(A[0]>A[1])
   {
      if(A[0]>A[2])  
      {
          i0=1;      /* A[0] is greatest */
          i1=2;
      }
      else
      {
          i0=0;      /* A[2] is greatest */
          i1=1;
      }
   }
   else   /* A[0]<=A[1] */
   {
      if(A[2]>A[1])
      {
          i0=0;      /* A[2] is greatest */
          i1=1;                                           
      }
      else
      {
          i0=0;      /* A[1] is greatest */
          i1=2;
      }
    }               
                
    /* test all edges of triangle 1 against the edges of triangle 2 */
    EDGE_AGAINST_TRI_EDGES(V0,V1,U0,U1,U2);
    EDGE_AGAINST_TRI_EDGES(V1,V2,U0,U1,U2);
    EDGE_AGAINST_TRI_EDGES(V2,V0,U0,U1,U2);
                
    /* finally, test if tri1 is totally contained in tri2 or vice versa */
    POINT_IN_TRI(V0,U0,U1,U2);
    POINT_IN_TRI(U0,V0,V1,V2);

    return 0;
}

/* sort so that a<=b */
#define SORT2(a,b,smallest)       \
             if(a>b)       \
             {             \
               double c;    \
               c=a;        \
               a=b;        \
               b=c;        \
               smallest=1; \
             }             \
             else smallest=0;


void isect2(double VTX0[3],double VTX1[3],double VTX2[3],double VV0,double VV1,double VV2,
	    double D0,double D1,double D2,double *isect0,double *isect1,double isectpoint0[3],double isectpoint1[3])
{
  double tmp=D0/(D0-D1);          
  double diff[3];
  *isect0=VV0+(VV1-VV0)*tmp;         
  SUB(diff,VTX1,VTX0);              
  MULT(diff,diff,tmp);               
  ADD(isectpoint0,diff,VTX0);        
  tmp=D0/(D0-D2);                    
  *isect1=VV0+(VV2-VV0)*tmp;          
  SUB(diff,VTX2,VTX0);                   
  MULT(diff,diff,tmp);                 
  ADD(isectpoint1,VTX0,diff);          
}


#if 0
#define ISECT2(VTX0,VTX1,VTX2,VV0,VV1,VV2,D0,D1,D2,isect0,isect1,isectpoint0,isectpoint1) \
              tmp=D0/(D0-D1);                    \
              isect0=VV0+(VV1-VV0)*tmp;          \
	      SUB(diff,VTX1,VTX0);               \
	      MULT(diff,diff,tmp);               \
              ADD(isectpoint0,diff,VTX0);        \ 
              tmp=D0/(D0-D2);                    
/*              isect1=VV0+(VV2-VV0)*tmp;          \ */
/*              SUB(diff,VTX2,VTX0);               \     */
/*              MULT(diff,diff,tmp);               \   */
/*              ADD(isectpoint1,VTX0,diff);           */
#endif

int compute_intervals_isectline(double VERT0[3],double VERT1[3],double VERT2[3],
				       double VV0,double VV1,double VV2,double D0,double D1,double D2,
				       double D0D1,double D0D2,double *isect0,double *isect1,
				       double isectpoint0[3],double isectpoint1[3])
{
  if(D0D1>0.0f)                                        
  {                                                    
    /* here we know that D0D2<=0.0 */                  
    /* that is D0, D1 are on the same side, D2 on the other or on the plane */
    isect2(VERT2,VERT0,VERT1,VV2,VV0,VV1,D2,D0,D1,isect0,isect1,isectpoint0,isectpoint1);
  } 
  else if(D0D2>0.0f)                                   
    {                                                   
    /* here we know that d0d1<=0.0 */             
    isect2(VERT1,VERT0,VERT2,VV1,VV0,VV2,D1,D0,D2,isect0,isect1,isectpoint0,isectpoint1);
  }                                                  
  else if(D1*D2>0.0f || D0!=0.0f)   
  {                                   
    /* here we know that d0d1<=0.0 or that D0!=0.0 */
    isect2(VERT0,VERT1,VERT2,VV0,VV1,VV2,D0,D1,D2,isect0,isect1,isectpoint0,isectpoint1);   
  }                                                  
  else if(D1!=0.0f)                                  
  {                                               
    isect2(VERT1,VERT0,VERT2,VV1,VV0,VV2,D1,D0,D2,isect0,isect1,isectpoint0,isectpoint1); 
  }                                         
  else if(D2!=0.0f)                                  
  {                                                   
    isect2(VERT2,VERT0,VERT1,VV2,VV0,VV1,D2,D0,D1,isect0,isect1,isectpoint0,isectpoint1);     
  }                                                 
  else                                               
  {                                                   
    /* triangles are coplanar */    
    return 1;
  }
  return 0;
}

#define COMPUTE_INTERVALS_ISECTLINE(VERT0,VERT1,VERT2,VV0,VV1,VV2,D0,D1,D2,D0D1,D0D2,isect0,isect1,isectpoint0,isectpoint1) \
  if(D0D1>0.0f)                                         \
  {                                                     \
    /* here we know that D0D2<=0.0 */                   \
    /* that is D0, D1 are on the same side, D2 on the other or on the plane */ \
    isect2(VERT2,VERT0,VERT1,VV2,VV0,VV1,D2,D0,D1,&isect0,&isect1,isectpoint0,isectpoint1);          \
  }                                                     
#if 0
  else if(D0D2>0.0f)                                    \
  {                                                     \
    /* here we know that d0d1<=0.0 */                   \
    isect2(VERT1,VERT0,VERT2,VV1,VV0,VV2,D1,D0,D2,&isect0,&isect1,isectpoint0,isectpoint1);          \
  }                                                     \
  else if(D1*D2>0.0f || D0!=0.0f)                       \
  {                                                     \
    /* here we know that d0d1<=0.0 or that D0!=0.0 */   \
    isect2(VERT0,VERT1,VERT2,VV0,VV1,VV2,D0,D1,D2,&isect0,&isect1,isectpoint0,isectpoint1);          \
  }                                                     \
  else if(D1!=0.0f)                                     \
  {                                                     \
    isect2(VERT1,VERT0,VERT2,VV1,VV0,VV2,D1,D0,D2,&isect0,&isect1,isectpoint0,isectpoint1);          \
  }                                                     \
  else if(D2!=0.0f)                                     \
  {                                                     \
    isect2(VERT2,VERT0,VERT1,VV2,VV0,VV1,D2,D0,D1,&isect0,&isect1,isectpoint0,isectpoint1);          \
  }                                                     \
  else                                                  \
  {                                                     \
    /* triangles are coplanar */                        \
    coplanar=1;                                         \
    return coplanar_tri_tri(N1,V0,V1,V2,U0,U1,U2);      \
  }
#endif

int tri_tri_intersect_with_isectline(double V0[3],double V1[3],double V2[3],
				     double U0[3],double U1[3],double U2[3],int *coplanar,
				     double isectpt1[3],double isectpt2[3])
{
  double E1[3],E2[3];
  double N1[3],N2[3],d1,d2;
  double du0,du1,du2,dv0,dv1,dv2;
  double D[3];
  double isect1[2], isect2[2];
  double isectpointA1[3],isectpointA2[3];
  double isectpointB1[3],isectpointB2[3];
  double du0du1,du0du2,dv0dv1,dv0dv2;
  short index;
  double vp0,vp1,vp2;
  double up0,up1,up2;
  double b,c,max;
//  double tmp,diff[3];
  int smallest1,smallest2;
  
  /* compute plane equation of triangle(V0,V1,V2) */
  SUB(E1,V1,V0);
  SUB(E2,V2,V0);
  CROSS(N1,E1,E2);
  d1=-DOT(N1,V0);
  /* plane equation 1: N1.X+d1=0 */

  /* put U0,U1,U2 into plane equation 1 to compute signed distances to the plane*/
  du0=DOT(N1,U0)+d1;
  du1=DOT(N1,U1)+d1;
  du2=DOT(N1,U2)+d1;

  /* coplanarity robustness check */
#if USE_EPSILON_TEST==TRUE
  if(fabs(du0)<EPSILON) du0=0.0;
  if(fabs(du1)<EPSILON) du1=0.0;
  if(fabs(du2)<EPSILON) du2=0.0;
#endif
  du0du1=du0*du1;
  du0du2=du0*du2;

  if(du0du1>0.0f && du0du2>0.0f) /* same sign on all of them + not equal 0 ? */
    return 0;                    /* no intersection occurs */

  /* compute plane of triangle (U0,U1,U2) */
  SUB(E1,U1,U0);
  SUB(E2,U2,U0);
  CROSS(N2,E1,E2);
  d2=-DOT(N2,U0);
  /* plane equation 2: N2.X+d2=0 */

  /* put V0,V1,V2 into plane equation 2 */
  dv0=DOT(N2,V0)+d2;
  dv1=DOT(N2,V1)+d2;
  dv2=DOT(N2,V2)+d2;

#if USE_EPSILON_TEST==TRUE
  if(fabs(dv0)<EPSILON) dv0=0.0;
  if(fabs(dv1)<EPSILON) dv1=0.0;
  if(fabs(dv2)<EPSILON) dv2=0.0;
#endif

  dv0dv1=dv0*dv1;
  dv0dv2=dv0*dv2;
        
  if(dv0dv1>0.0f && dv0dv2>0.0f) /* same sign on all of them + not equal 0 ? */
    return 0;                    /* no intersection occurs */

  /* compute direction of intersection line */
  CROSS(D,N1,N2);

  /* compute and index to the largest component of D */
  max=fabs(D[0]);
  index=0;
  b=fabs(D[1]);
  c=fabs(D[2]);
  if(b>max) max=b,index=1;
  if(c>max) max=c,index=2;

  /* this is the simplified projection onto L*/
  vp0=V0[index];
  vp1=V1[index];
  vp2=V2[index];
  
  up0=U0[index];
  up1=U1[index];
  up2=U2[index];

  /* compute interval for triangle 1 */
  *coplanar=compute_intervals_isectline(V0,V1,V2,vp0,vp1,vp2,dv0,dv1,dv2,
				       dv0dv1,dv0dv2,&isect1[0],&isect1[1],isectpointA1,isectpointA2);
  if(*coplanar) return coplanar_tri_tri(N1,V0,V1,V2,U0,U1,U2);     


  /* compute interval for triangle 2 */
  compute_intervals_isectline(U0,U1,U2,up0,up1,up2,du0,du1,du2,
			      du0du1,du0du2,&isect2[0],&isect2[1],isectpointB1,isectpointB2);

  SORT2(isect1[0],isect1[1],smallest1);
  SORT2(isect2[0],isect2[1],smallest2);

  if(isect1[1]<isect2[0] || isect2[1]<isect1[0]) return 0;

  /* at this point, we know that the triangles intersect */

  if(isect2[0]<isect1[0])
  {
    if(smallest1==0) { SET(isectpt1,isectpointA1); }
    else { SET(isectpt1,isectpointA2); }

    if(isect2[1]<isect1[1])
    {
      if(smallest2==0) { SET(isectpt2,isectpointB2); }
      else { SET(isectpt2,isectpointB1); }
    }
    else
    {
      if(smallest1==0) { SET(isectpt2,isectpointA2); }
      else { SET(isectpt2,isectpointA1); }
    }
  }
  else
  {
    if(smallest2==0) { SET(isectpt1,isectpointB1); }
    else { SET(isectpt1,isectpointB2); }

    if(isect2[1]>isect1[1])
    {
      if(smallest1==0) { SET(isectpt2,isectpointA2); }
      else { SET(isectpt2,isectpointA1); }      
    }
    else
    {
      if(smallest2==0) { SET(isectpt2,isectpointB2); }
      else { SET(isectpt2,isectpointB1); } 
    }
  }
  return 1;
}

/* int tri_tri_intersect_with_isectline(double V0[3],double V1[3],double V2[3],
				     double U0[3],double U1[3],double U2[3],int *coplanar,
				     double isectpt1[3],double isectpt2[3])
*/

int tri_segment_intersect_with_isectpoint(double V0[3],double V1[3],double V2[3],
										  double S0[3],double S1[3],
										  double isectpt[3], int *idx)
{
  double u[3],v[3],n[3]; // triangle vectors
  double dir[3], w0[3], w[3];// ray vectors
  double temp[3];
  double r, a, b;
  double d1;
  double du0,du1;
  double du0du1;
  
  /* compute plane equation of triangle(V0,V1,V2) */
  SUB(u,V1,V0);
  SUB(v,V2,V0);
  CROSS(n,u,v);
  d1=-DOT(n,V0);
  /* plane equation 1: n.X+d1=0 */

  /* put S0, S1 into plane equation 1 to compute signed distances to the plane*/
  du0=DOT(n,S0)+d1;
  du1=DOT(n,S1)+d1;
  
  /* coplanarity robustness check */
#if USE_EPSILON_TEST==TRUE
  if(fabs(du0)<EPSILON) du0=0.0;
  if(fabs(du1)<EPSILON) du1=0.0;
#endif
  du0du1=du0*du1;

  if(du0du1>0.0f) /* same sign on both of them + not equal 0 ? */
    return 0;                    /* no intersection occurs */

  if(MANHATTAN(n,null)<EPSILON) // triangle is degenerate
	return 0;

  *idx=(du0>0)?0:1; // Which segment end is "outside" triangle?

  SUB(dir, S1, S0);  // ray direction vector
  SUB(w0, S0, V0);
  a=-DOT(n,w0);
  b=DOT(n,dir);

  if (fabs(b) < EPSILON) {     // ray is parallel to triangle plane
     return 0;             
  }

  r = a / b;
  MULT(temp, dir, r);
  ADD(isectpt, S0, temp);

  // is I inside T?
    double uu, uv, vv, wu, wv, D1;
    uu = DOT(u,u);
    uv = DOT(u,v);
    vv = DOT(v,v);
    SUB(w, isectpt, V0); 
    wu = DOT(w,u);
    wv = DOT(w,v);
    D1 = uv * uv - uu * vv;

    // get and test parametric coords
    double s, t;
    s = (uv * wv - vv * wu) / D1;
    if (s < -EPSILON || s > 1.0+EPSILON)        // I is outside T
        return 0;
    t = (uv * wu - uu * wv) / D1;
    if (t < -EPSILON || (s + t) > 1.0+EPSILON)  // I is outside T
        return 0;

    return 1;                      // I is in T
}

// Check side of point over triangle
double Side(double P[3], double V0[3], double V1[3], double V2[3])
{
	double u[3],v[3],n[3]; // triangle vectors
	double d1;

	SUB(u,V1,V0);
	SUB(v,V2,V0);
	CROSS(n,u,v);
	d1=-DOT(n,V0);
	/* plane equation 1: n.X+d1=0 */
	
	/* put P into plane equation 1 to compute signed distances to the plane*/
	return DOT(n,P)+d1;
}

// Check if two coplanar triangles have the same winding
int SameWinding (double T1a[3], double T1b[3], double T1c[3], double T2a[3], double T2b[3], double T2c[3])
{
	double u1[3], v1[3], u2[3], v2[3];
	double n1[3], n2[3];
	SUB(u1, T1b, T1a);
	SUB(v1, T1c, T1a);
	SUB(u2, T2b, T2a);
	SUB(v2, T2c, T2a);
	CROSS(n1, u1, v1);
	CROSS(n2, u2, v2);
	if(DOT(n1,n2)>0) return true; 
	else return false;
}


// Calculate "flatness" of angle p1p2p3 (normalized!)
double flatness (double p1[3], double p2[3], double p3[3])
{
	double v[3], w[3], x[3];
	double l1, l2;
	double result;
	SUB(v,p2,p1);
	SUB(w,p2,p3);
	CROSS(x,v,w);
	l1=NORM(v);
	l2=NORM(w);
	MULT(x, x, (1./l1));
	MULT(x, x, (1./l2));
	result=NORM(x);
	return (result);
}

	
// dist_Point_to_Line(): get the distance of a point to a line.
//    Input:  a Point P and a Line L (in any dimension)
//    Return: the shortest distance from P to L

double dist_Point_to_Line( double P[3], double L1[3], double L2[3])
{
	double v[3], w[3], Pb[3];
	SUB (v, L2, L1);
	SUB (w, P, L1);

    double c1 = DOT(w,v);
    double c2 = DOT(v,v);
    double b = c1 / c2;

	MULT (w,v,b);
	ADD (Pb, L1, w);
    return DIST(P, Pb);
}

// Check if a quad is convex and don't have aligned vertexes.

int Convex(double U0[3],double U1[3],double U2[3],double U3[3])
{
	double U01[3], U12[3], U23[3], U30[3];
	double T0[3], T1[3], T2[3], T3[3];
	int cnt=0;
	SUB(U01, U1, U0);
	SUB(U12, U2, U1);
	SUB(U23, U3, U2);
	SUB(U30, U0, U3);

	CROSS(T0, U30, U01);
	CROSS(T1, U01, U12);
	CROSS(T2, U12, U23);
	CROSS(T3, U23, U30);

	// Colinear Vertices
	// if((DOT(T0, T0)<EPSILON) ||(DOT(T1, T1)<EPSILON) ||(DOT(T2, T2)<EPSILON) ||(DOT(T3, T3)<EPSILON)) return false;

	if(DOT(T0, T1) > 0) cnt++; else cnt--;
	if(DOT(T1, T2) > 0) cnt++; else cnt--;
	if(DOT(T2, T3) > 0) cnt++; else cnt--;
	if(DOT(T3, T0) > 0) cnt++; else cnt--;

	if(abs(cnt) == 4) return true;
	else return false;
}

void usage(void)
{
	
	printf ("Usage: Intersector [-c] [-t] [-s scale] LdrawCutFile LdrawCutterFile LdrawCutFileOut\n");
	printf ("Options\n");
	printf ("  -c: coloring of cut elements according to orientation of cutter surface\n");
	printf ("  -t: no triangle condensing after cutting\n");
	printf ("  -s: does a prescaling by factor <scale> before intersection\n");
	printf ("      Use this for tiny parts. Size is restored in the end\n");
	printf("Press <Enter> to quit");
	getchar();
}

#define POLYTABSIZE 100
double polytab [POLYTABSIZE][3];


// Rotates the vertexes of polytab and delete "del" vertexes
void rotpoly(int size, int del)
{
	double temp[3];
	int i;
	SET(temp, polytab[0]);
	for(i=del+1; i<size; i++)
	{
		SET(polytab[i-1-del], polytab[i]);
	}
	SET(polytab[i-1-del], temp);
}

#define SMALL 0.0002

struct LDrawLine
{
	int type;
	int color;
	double a[3];
	double b[3];
	double c[3];
	double d[3];
	int index;
	int mod;
	int side;
};

#define LDRAWCPY(dest,src)	\
{							\
	SET(dest.a, src.a);		\
	SET(dest.b, src.b);		\
	SET(dest.c, src.c);		\
	SET(dest.d, src.d);		\
	dest.type = src.type;	\
	dest.color = src.color;	\
	dest.index = src.index;	\
	dest.mod = src.mod;		\
	dest.side = src.side;	\
}

	LDrawLine *In;
	LDrawLine *Cut;

int main(int argc, char* argv[])
{
	double isectpt[3];
	int idx;

	FILE *LdrawIn;
	FILE *LdrawCutter;
	FILE *LdrawOut;

	char buf[1024];
	int type;

	int NumIn, NumCut;
	int NumInLast;
	int NumInMax;

	int i,j,ltype;

	int n[6];


	int coloring=false;
	int tricondens=true;
	double prescaling=1.;
	double degen;

	srand ( time(NULL) );
	printf("\nLdraw Intersector v1.4 - by Philo");
	printf("\n---------------------------------\n");

	if(argc < 4) 		
	{
		usage();
		return 1;
	}

	for(i=1; i<argc-3; i++)
	{
		if(argv[i][0] != '-') 
		{
			usage();
			return 1;
		}
		switch (argv[i][1])
		{
		case 'c':
			coloring = true;
			break;
		case 't':
			tricondens = false;
			break;
		case 's':
			i++;
			sscanf(argv[i], "%lf", &prescaling);
			if (prescaling<=0)
			{
				printf("Length (-l %lf) must be > 0\n", prescaling);
				printf("Press <Enter> to quit");
				getchar();
				return 3;
			}
			printf("prescaling = %lf\n", prescaling);
			break;
		default:
			usage();
			return 1;
		}
	}


	LdrawIn = fopen (argv[argc-3], "rt");
	if(LdrawIn == NULL)
	{
		printf("Can't open file\n%s\n", argv[argc-3]);
		printf("Press <Enter> to quit");
		getchar();
		return 2;
	}

	LdrawCutter = fopen (argv[argc-2], "rt");
	if(LdrawCutter == NULL)
	{
		printf("Can't open file\n%s\n", argv[argc-2]);
		printf("Press <Enter> to quit");
		getchar();
		return 2;
	}

	LdrawOut = fopen (argv[argc-1], "wt");
	if(LdrawOut == NULL)
	{
		printf("Can't create file\n%s\n", argv[argc-1]);
		printf("Press <Enter> to quit");
		getchar();
		return 2;
	}

	
	
	printf("\nCounting Input Cut File...\n");
	NumIn=0;
	// Get number of elements of cutfile
	while (fgets(buf, 1024, LdrawIn)) 
	{
		type = -1;
		sscanf(buf, "%d", &type);
		switch(type)
		{
		case 2:
		case 3:
		case 5:
			NumIn++;
			break;
		case 4:
			NumIn+=2;
			break;
		default:
			break;
		}
	}
	printf("  %d elements in cut file\n", NumIn);
	rewind(LdrawIn);

	printf("\nCounting Input Cutter File...\n");
	NumCut=0;
	// Get number of elements of cutterfile
	while (fgets(buf, 1024, LdrawCutter)) 
	{
		type = -1;
		sscanf(buf, "%d", &type);
		switch(type)
		{
		case 3:
			NumCut++;
			break;
		case 4:
			NumCut+=2;
			break;
		default:
			break;
		}
	}
	printf("  %d elements in cutter file\n", NumCut);
	rewind(LdrawCutter);

	int Maxin = (NumIn+NumCut+10000)*2;

	In=(LDrawLine*)malloc(Maxin*sizeof(LDrawLine));
	if(In==NULL)
	{
		printf("Can't allocate memory\n");
		printf("Press <Enter> to quit");
		getchar();
		return 2;
	}

	int Maxcut=NumCut;

	Cut=(LDrawLine*)malloc(Maxcut*sizeof(LDrawLine));
	if(In==NULL)
	{
		printf("Can't allocate memory\n");
		printf("Press <Enter> to quit");
		getchar();
		return 2;
	}


	printf("\nReading Input Cut File...\n");
	NumIn=0;
	while (fgets(buf, 1024, LdrawIn))
	{
		type = -1;
		sscanf(buf, "%d", &type);
		
		switch(type)
		{
		case 2:
			sscanf (buf, "%d %d %lf %lf %lf %lf %lf %lf", &In[NumIn].type, &In[NumIn].color, 
				&In[NumIn].a[0], &In[NumIn].a[1], &In[NumIn].a[2], 
				&In[NumIn].b[0], &In[NumIn].b[1], &In[NumIn].b[2]);
			In[NumIn].side=0;
			In[NumIn].mod=false;
			In[NumIn].index=NumIn;
			NumIn++;
			break;
		case 3:
			sscanf (buf, "%d %d %lf %lf %lf %lf %lf %lf %lf %lf %lf", &In[NumIn].type, &In[NumIn].color, 
				&In[NumIn].a[0], &In[NumIn].a[1], &In[NumIn].a[2], 
				&In[NumIn].b[0], &In[NumIn].b[1], &In[NumIn].b[2],
				&In[NumIn].c[0], &In[NumIn].c[1], &In[NumIn].c[2]);
			In[NumIn].side=0;
			In[NumIn].mod=false;
			In[NumIn].index=NumIn;

			NumIn++;
			break;
		case 4:
		case 5:
			sscanf (buf, "%d %d %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf", &In[NumIn].type, &In[NumIn].color, 
				&In[NumIn].a[0], &In[NumIn].a[1], &In[NumIn].a[2], 
				&In[NumIn].b[0], &In[NumIn].b[1], &In[NumIn].b[2],
				&In[NumIn].c[0], &In[NumIn].c[1], &In[NumIn].c[2],
				&In[NumIn].d[0], &In[NumIn].d[1], &In[NumIn].d[2]);
			In[NumIn].side=0;
			In[NumIn].mod=false;
			In[NumIn].index=NumIn;
			if(In[NumIn].type == 4)
			{
				In[NumIn].type = 3;
				SET(In[NumIn+1].a,In[NumIn].a);
				SET(In[NumIn+1].b,In[NumIn].c);
				SET(In[NumIn+1].c,In[NumIn].d);
				NumIn++;
				In[NumIn].type = 3;
				In[NumIn].color=In[NumIn-1].color;
				In[NumIn].side=0;
				In[NumIn].mod=false;
				In[NumIn].index=In[NumIn-1].index;
			}

			NumIn++;
			break;
		default :
			break;
		}

		if(NumIn>Maxin)
		{
			printf("Too many lines in cut file\n");
			printf("Press <Enter> to quit");
			getchar();
			return 3;
		}

	}
	NumInMax=NumIn;	

	// Prescaling
	for(i=0; i<NumInMax; i++)
	{
		MULT(In[i].a, In[i].a, prescaling);
		MULT(In[i].b, In[i].b, prescaling);
		MULT(In[i].c, In[i].c, prescaling);
		MULT(In[i].d, In[i].d, prescaling);
	}

	printf("Reading Cutter File...\n");
	NumCut=0;
	while (fgets(buf, 1024, LdrawCutter))
	{
		type = -1;
		sscanf(buf, "%d", &type);
		
		switch(type)
		{
		case 3:
			sscanf (buf, "%d %d %lf %lf %lf %lf %lf %lf %lf %lf %lf", &Cut[NumCut].type, &Cut[NumCut].color, 
				&Cut[NumCut].a[0], &Cut[NumCut].a[1], &Cut[NumCut].a[2], 
				&Cut[NumCut].b[0], &Cut[NumCut].b[1], &Cut[NumCut].b[2],
				&Cut[NumCut].c[0], &Cut[NumCut].c[1], &Cut[NumCut].c[2]);
			Cut[NumCut].side=0;
			Cut[NumCut].mod=false;
			Cut[NumCut].index=NumCut;

			NumCut++;
			break;
		case 4:
			sscanf (buf, "%d %d %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf", &Cut[NumCut].type, &Cut[NumCut].color, 
				&Cut[NumCut].a[0], &Cut[NumCut].a[1], &Cut[NumCut].a[2], 
				&Cut[NumCut].b[0], &Cut[NumCut].b[1], &Cut[NumCut].b[2],
				&Cut[NumCut].c[0], &Cut[NumCut].c[1], &Cut[NumCut].c[2],
				&Cut[NumCut].d[0], &Cut[NumCut].d[1], &Cut[NumCut].d[2]);
			Cut[NumCut].side=0;
			Cut[NumCut].mod=false;
			Cut[NumCut].index=NumCut;

			Cut[NumCut].type = 3;
			SET(Cut[NumCut+1].a,Cut[NumCut].a);
			SET(Cut[NumCut+1].b,Cut[NumCut].c);
			SET(Cut[NumCut+1].c,Cut[NumCut].d);
			NumCut++;
			Cut[NumCut].type = 3;
			Cut[NumCut].color=Cut[NumCut-1].color;
			Cut[NumCut].side=0;
			Cut[NumCut].mod=false;
			Cut[NumCut].index=Cut[NumCut-1].index;

			NumCut++;
			break;
		default :
			break;
		}

		if(NumCut>Maxcut)
		{
			printf("Too many lines in cutter file\n");
			printf("Press <Enter> to quit");
			getchar();
			return 3;
		}

	}
	printf("  %d element(s) in cutter file\n", NumCut);
	// Prescaling
	for(i=0; i<NumCut; i++)
	{
		MULT(Cut[i].a, Cut[i].a, prescaling);
		MULT(Cut[i].b, Cut[i].b, prescaling);
		MULT(Cut[i].c, Cut[i].c, prescaling);
		MULT(Cut[i].d, Cut[i].d, prescaling);
	}


	printf("Cutting things...\n");

	for(i=0; i<NumInMax; i++)
	{
		if(NumInMax>Maxin-4)
		{
			printf("Too many intersections found\n");
			printf("Press <Enter> to quit");
			getchar();
			return 3;
		}

		switch(In[i].type)
		{
		case 2:
		case 5:
			for(j=0; j<NumCut; j++)
			{
				if(tri_segment_intersect_with_isectpoint(Cut[j].a, Cut[j].b, Cut[j].c, In[i].a, In[i].b, isectpt, &idx))
				{
					if((MANHATTAN(In[i].a, isectpt)<SMALL) || (MANHATTAN(In[i].b, isectpt)<SMALL)) continue;
					
					LDRAWCPY(In[NumInMax], In[i]);
					In[NumInMax].mod = In[i].mod = true;
					SET(In[i].a, isectpt);
					SET(In[NumInMax].b, isectpt);
					if(idx==0)
					{
						In[i].side=-1;
						In[NumInMax].side=1;
					}
					else
					{
						In[i].side=1;
						In[NumInMax].side=-1;
					}
					NumInMax++;
				}
			}
			break;
		case 3:
			for(j=0; j<NumCut; j++)
			{
				int coplanar;
				int vertexaligned=false;
				int isectptonedge=false;
				int secondonedge=false;
				double isectpt1[3], isectpt2[3];
				// calculate intersection
				if(tri_tri_intersect_with_isectline(In[i].a, In[i].b, In[i].c, Cut[j].a, Cut[j].b, Cut[j].c,
					&coplanar, isectpt1, isectpt2))
				{
					// intersection not empty, and not degenerated
					if(coplanar) continue;
					// Check if intersection is reduced to a single point. Nothing to do then.
					if(DIST(isectpt1, isectpt2)<SMALL) continue;
					// 1) intersection line is aligned with one triangle vertex
					//    Triangle points are reordered so point a is the one aligned
					if(dist_Point_to_Line(In[i].a, isectpt1, isectpt2) < SMALL) vertexaligned=true;
					if(dist_Point_to_Line(In[i].b, isectpt1, isectpt2) < SMALL) 
					{
						vertexaligned=true;
						ROTINV(In[i].a, In[i].b, In[i].c);
					}
					if(dist_Point_to_Line(In[i].c, isectpt1, isectpt2) < SMALL) 
					{
						vertexaligned=true;
						ROTDIR(In[i].a, In[i].b, In[i].c);
					}
					if(vertexaligned)
					{
						// 1a) Check if intersection line is aligned with triangle edge. Nothing to do then.
						//     except check triangle side.
						if(dist_Point_to_Line(In[i].b, isectpt1, isectpt2) < SMALL) 
						{
							if(Side(In[i].c, Cut[j].a, Cut[j].b, Cut[j].c) < 0)
							{
								In[i].side=-1;
								In[NumInMax].side=1;
							}
							else
							{
								In[i].side=1;
								In[NumInMax].side=-1;
							}
							continue;
						}

						if(dist_Point_to_Line(In[i].c, isectpt1, isectpt2) < SMALL)
						{
							if(Side(In[i].c, Cut[j].a, Cut[j].b, Cut[j].c) < 0)
							{
								In[i].side=-1;
								In[NumInMax].side=1;
							}
							else
							{
								In[i].side=1;
								In[NumInMax].side=-1;
							}
							continue;
						}


						// 1b) Check if one point is on opposite edge. Simple split in that case.
						if(dist_Point_to_Line(In[i].b, In[i].c, isectpt2) < SMALL) SWAP(isectpt1, isectpt2);
						if(dist_Point_to_Line(In[i].b, In[i].c, isectpt1) < SMALL) 
						{
							LDRAWCPY(In[NumInMax], In[i]);
							In[NumInMax].mod = In[i].mod = true;
							SET(In[i].c, isectpt1);
							SET(In[NumInMax].b, isectpt1);
							if(Side(In[i].b, Cut[j].a, Cut[j].b, Cut[j].c) < 0)
							{
								In[i].side=-1;
								In[NumInMax].side=1;
							}
							else
							{
								In[i].side=1;
								In[NumInMax].side=-1;
							}
							NumInMax++;
							continue;
						}
						// 1c) One point is in the middle of the triangle. Cut triangle in three parts.
						// Find point further from vertex
						if (DIST(In[i].a, isectpt1) > DIST(In[i].a, isectpt2)) SWAP(isectpt1, isectpt2);

						LDRAWCPY(In[NumInMax], In[i]);
						LDRAWCPY(In[NumInMax+1], In[i]);
						In[NumInMax+1].mod = In[NumInMax].mod = In[i].mod = true;
						SET(In[i].a, isectpt2);
						SET(In[NumInMax].c, isectpt2);
						SET(In[NumInMax+1].b, isectpt2);
						In[i].side=0;
						if(Side(In[i].b, Cut[j].a, Cut[j].b, Cut[j].c) < 0)
						{
							In[NumInMax].side=-1;
							In[NumInMax+1].side=1;
						}
						else
						{
							In[NumInMax].side=1;
							In[NumInMax+1].side=-1;
						}
						NumInMax+=2;
					}
					else
					{
						// 2) One point is on triangle edge. 
						//    Triangle is reordered so that vertex a is opposite to that edge
						//    Line is reordered so that isectp1 is on edge bc
						if (dist_Point_to_Line(In[i].b, In[i].c, isectpt1) < SMALL)
							isectptonedge=true;
						else if (dist_Point_to_Line(In[i].b, In[i].c, isectpt2) < SMALL)
						{
							isectptonedge=true;
							SWAP (isectpt1, isectpt2);
						}
						else if (dist_Point_to_Line(In[i].a, In[i].b, isectpt1) < SMALL)
						{
							isectptonedge=true;
							ROTDIR(In[i].a, In[i].b, In[i].c);
						}
						else if (dist_Point_to_Line(In[i].a, In[i].b, isectpt2) < SMALL)
						{	isectptonedge=true;
							ROTDIR(In[i].a, In[i].b, In[i].c);
							SWAP (isectpt1, isectpt2);
						}
						else if (dist_Point_to_Line(In[i].a, In[i].c, isectpt1) < SMALL)
						{
							isectptonedge=true;
							ROTINV(In[i].a, In[i].b, In[i].c);
						}
						else if (dist_Point_to_Line(In[i].a, In[i].c, isectpt2) < SMALL)
						{	isectptonedge=true;
							ROTINV(In[i].a, In[i].b, In[i].c);
							SWAP (isectpt1, isectpt2);
						}
						if(isectptonedge)
						{
							// 2a) The second point is also on a triangle edge.
							//     Triangle is reordered so that a is separated from b and c by intersection line
							//     The triangle will be split in three parts
							// 
							if (dist_Point_to_Line(In[i].a, In[i].c, isectpt2) < SMALL)
							{
								ROTDIR(In[i].a, In[i].b, In[i].c);
								SWAP (isectpt1, isectpt2);
								secondonedge=true;
							}
							else if (dist_Point_to_Line(In[i].a, In[i].b, isectpt2) < SMALL)
							{
								ROTINV(In[i].a, In[i].b, In[i].c);
								secondonedge=true;
							}
							if(secondonedge)
							{
								LDRAWCPY(In[NumInMax], In[i]);
								LDRAWCPY(In[NumInMax+1], In[i]);
								In[NumInMax+1].mod = In[NumInMax].mod = In[i].mod = true;
								SET(In[i].b, isectpt1);
								SET(In[i].c, isectpt2);

								if(DIST(isectpt1, In[i].c) < DIST(isectpt2, In[i].b))
								{
									SET(In[NumInMax].a, isectpt2);
									SET(In[NumInMax].b, isectpt1);
									SET(In[NumInMax+1].a, isectpt1);
								}
								else
								{
									SET(In[NumInMax].a, isectpt1);
									SET(In[NumInMax].c, isectpt2);
									SET(In[NumInMax+1].a, isectpt2);
								}

								if(Side(In[i].a, Cut[j].a, Cut[j].b, Cut[j].c) < 0)
								{
									In[i].side=-1;
									In[NumInMax].side=1;
									In[NumInMax+1].side=1;
								}
								else
								{
									In[i].side=1;
									In[NumInMax].side=-1;
									In[NumInMax+1].side=-1;
								}
								NumInMax+=2;  
								continue;
							}
							else
							{
							// 2b) The second point is in the middle of the triangle
							//     The triangle will be split in four parts
								LDRAWCPY(In[NumInMax], In[i]);
								LDRAWCPY(In[NumInMax+1], In[i]);
								LDRAWCPY(In[NumInMax+2], In[i]);
								In[NumInMax+2].mod = In[NumInMax+1].mod = In[NumInMax].mod = In[i].mod = true;
								SET(In[i].c, isectpt2);
								SET(In[NumInMax].b, isectpt2);
								SET(In[NumInMax+1].a, isectpt2);
								SET(In[NumInMax+2].a, isectpt2);
								SET(In[NumInMax+1].c, isectpt1);
								SET(In[NumInMax+2].b, isectpt1);
								if(Side(In[i].b, Cut[j].a, Cut[j].b, Cut[j].c) < 0)
								{
									if(Side(In[i].a, Cut[j].a, Cut[j].b, Cut[j].c) < 0)
									{
										In[i].side=-1;
										In[NumInMax].side=0;
									}
									else
									{
										In[i].side=0;
										In[NumInMax].side=1;
									}
									In[NumInMax+1].side=-1;
									In[NumInMax+2].side=1;
								}
								else
								{
									if(Side(In[i].a, Cut[j].a, Cut[j].b, Cut[j].c) > 0)
									{
										In[i].side=1;
										In[NumInMax].side=0;
									}
									else
									{
										In[i].side=0;
										In[NumInMax].side=-1;
									}
									In[NumInMax+1].side=1;
									In[NumInMax+2].side=-1;

								}
								NumInMax+=3;
								continue;
							}
						}
						else
						{
						// 3) Both intersection points are inside triangle.
							//     The triangle will be split in five parts

							// Find the edge of the triangle with both ends on the same side from intersection triangle
							if(Side(In[i].a, Cut[j].a, Cut[j].b, Cut[j].c) * Side(In[i].b, Cut[j].a, Cut[j].b, Cut[j].c) > 0)
							{
								ROTDIR(In[i].a, In[i].b, In[i].c);
							}
							else if(Side(In[i].a, Cut[j].a, Cut[j].b, Cut[j].c) * Side(In[i].c, Cut[j].a, Cut[j].b, Cut[j].c) > 0)
							{
								ROTINV(In[i].a, In[i].b, In[i].c);
							}
							// intersection is now more or less along bc
							if(dist_Point_to_Line(isectpt2,In[i].a, In[i].b) < dist_Point_to_Line(isectpt1,In[i].a, In[i].b)) SWAP (isectpt1, isectpt2);
							for(int k=0; k<4; k++) LDRAWCPY(In[NumInMax+k], In[i]);
							In[NumInMax+3].mod = In[NumInMax+2].mod = In[NumInMax+1].mod = In[NumInMax].mod = In[i].mod = true;
							SET(In[i].b, isectpt1);
							SET(In[i].c, isectpt2);
							SET(In[NumInMax].c, isectpt1);
							SET(In[NumInMax+1].b, isectpt2);
							if(DIST(isectpt1, In[i].c) > DIST(isectpt2, In[i].b))
							{
								SET(In[NumInMax+2].a, isectpt2);
								SET(In[NumInMax+3].a, isectpt1);
								SET(In[NumInMax+3].c, isectpt2);
							}
							else
							{
								SET(In[NumInMax+2].a, isectpt1);
								SET(In[NumInMax+3].a, isectpt2);
								SET(In[NumInMax+3].b, isectpt1);
							}

							In[NumInMax].side=0;
							In[NumInMax+1].side=0;
							if(Side(In[i].a, Cut[j].a, Cut[j].b, Cut[j].c) < 0)
							{
								In[i].side=-1;
								In[NumInMax+2].side=1;
								In[NumInMax+3].side=1;
							}
							else
							{
								In[i].side=1;
								In[NumInMax+2].side=-1;
								In[NumInMax+3].side=-1;
							}
							NumInMax+=4;
						}
					}
				}
			}
			j=j;
			break;
		default:
			break;
		}
	}

	// Suppressing degenerated triangles.
	for(i=0; i<NumInMax; i++)
	{
		if((dist_Point_to_Line(In[i].a, In[i].b, In[i].c) < SMALL) || (dist_Point_to_Line(In[i].b, In[i].c, In[i].a) < SMALL) || (dist_Point_to_Line(In[i].c, In[i].b, In[i].a) < SMALL))
		{
			In[i].index =-1;
			continue;
		}	
	}

	if(coloring)
	{
		printf("Coloring according to side...\n");
		for(i=0; i<NumInMax; i++)
		{
			switch(In[i].type)
			{
			case 2:
			case 5:
				switch(In[i].side)
				{
				case 1:
					In[i].color=1;
					break;
				case 0:
					In[i].color=0;
					break;
				case -1:
					In[i].color=28;
					break;
				}
			break;
			case 3:
				switch(In[i].side)
				{
				case 1:
					In[i].color=11;
					break;
				case 0:
					In[i].color=7;
					break;
				case -1:
					In[i].color=14;
					break;
				}
			break;
			default:
				break;
			}
		}
	}

	if(tricondens)
	{
		
		printf("Condensing and retriangulating same side polygons...\n");

		// This assumes that after cut all the triangles from the same surface have the same winding.

		NumInLast=NumInMax;
		for(i=0; i<NumInMax; i++)
		{
			int k,l,n;
			int condense;
			int polmax;
			int retri;
			int instart;

			if(In[i].index < 0) continue;
			if(In[i].type !=3) continue;
			if(!In[i].mod) continue;
			SET (polytab[0], In[i].a); 
			SET (polytab[1], In[i].b); 
			SET (polytab[2], In[i].c);
			polmax=3;
			do{
				condense = false;
				for(j=i+1; j<NumInMax; j++)
				{
					if(In[j].index != In[i].index) continue;
					if(In[j].type !=3) continue;
					if(In[i].side != In[j].side) continue;

					for(k=0; k<polmax; k++) 
					{
						if(polmax >= POLYTABSIZE)
						{
							printf("Too many triangles in one facet\n");
							printf("Press <Enter> to quit");
							getchar();
							return 3;
						}
						for(l=0; l<3; l++)
						{
							if((DIST(polytab[0], In[j].a) < SMALL) && (DIST(polytab[polmax-1], In[j].b) < SMALL))
							{
								if(dist_Point_to_Line(polytab[0], polytab[1], In[j].c) < SMALL)
								{
									SET(polytab[0], In[j].c);
									In[j].index = -1;
									condense = true;
									if(dist_Point_to_Line(polytab[polmax-1], polytab[polmax-2], In[j].c) < SMALL) polmax--;
									break;
								} 
								else if(dist_Point_to_Line(polytab[polmax-1], polytab[polmax-2], In[j].c) < SMALL)
								{
									SET(polytab[polmax-1], In[j].c);
									In[j].index = -1;
									condense = true;
									break;
								}
								else
								SET(polytab[polmax], In[j].c);
								polmax++;
								In[j].index = -1;
								condense = true;
								break;
							}
							ROTINV(In[j].a, In[j].b, In[j].c);
						}
						if(condense) break;
						rotpoly(polmax,0);
					}
				}
			} while(condense);
			instart=NumInMax;
			// In[i].color=i; // Debug !

			for(j=0;j<(rand() % polmax); j++)
			{
				rotpoly(polmax,0);	
			}
			do 
			{
				retri=false;
				for(j=0; j<polmax; j++)
				{
					if(NumInLast >= Maxin)
					{
						printf("Too many triangles during retriangulation\n");
						printf("Press <Enter> to quit");
						getchar();
						return 3;
					}					
					if(SameWinding(In[i].a, In[i].b, In[i].c, polytab[0], polytab[1], polytab[2]))
					{
						int k, pointinside=false;
						for(k=3; k<polmax; k++)
						{
							if(SameWinding(In[i].a, In[i].b, In[i].c, polytab[1], polytab[2], polytab[k]) && !Convex(polytab[0], polytab[1], polytab[2], polytab[k]))
							{
								rotpoly(polmax, 0);
								pointinside=true;
								break;
							}
						}
						if(!pointinside)
						{
							if((polmax>3) && Convex(polytab[0], polytab[1], polytab[2], polytab[3]))
							{
								for(k=4; k<polmax; k++)
								{
									if( !Convex(polytab[0], polytab[2], polytab[3], polytab[k]))
									{
										pointinside=true;
										break;
									}
								}
								if(!pointinside)
								{
									LDRAWCPY(In[NumInLast], In[i]);
									SET (In[NumInLast].a, polytab[0]) ;
									SET (In[NumInLast].b, polytab[1]) ;
									SET (In[NumInLast].c, polytab[2]) ;
									SET (In[NumInLast].d, polytab[3]) ;
									In[NumInLast].type=4;
									rotpoly(polmax, 2);
									polmax-=2;
									NumInLast++;
									retri=true;
									break;
								}
								else
								{
									LDRAWCPY(In[NumInLast], In[i]);
									SET (In[NumInLast].a, polytab[0]) ;
									SET (In[NumInLast].b, polytab[1]) ;
									SET (In[NumInLast].c, polytab[2]) ;
									rotpoly(polmax, 1);
									polmax--;
									NumInLast++;
									retri=true;
									break;
								}
							}
						
							LDRAWCPY(In[NumInLast], In[i]);
							SET (In[NumInLast].a, polytab[0]) ;
							SET (In[NumInLast].b, polytab[1]) ;
							SET (In[NumInLast].c, polytab[2]) ;
							rotpoly(polmax, 1);
							polmax--;
							NumInLast++;
							retri=true;
							break;
						}
					}
					else
					{
						rotpoly(polmax, 0);
					}
				}
				if(polmax<3) break;
			} while (retri);

			if(polmax!=2) printf("\n*** problem during retriangulation *** %d \n", polmax);
			for(n=instart; n<NumInLast; n++)
			{
				if(In[n].type !=3) continue;
				if(In[n].index != In[i].index) continue;
				do{
					condense = false;
					for(j=n+1; j<NumInLast; j++)
					{
						if(In[j].type !=3) continue;
						if(In[j].index != In[i].index) continue;
						if(In[n].side != In[j].side) continue;

						for(k=0; k<3; k++)
						{
							for(l=0; l<3; l++)
							{
								if((DIST(In[n].a, In[j].a) < SMALL) && (DIST(In[n].c, In[j].b) < SMALL))
								{
									if(dist_Point_to_Line(In[n].a, In[n].b, In[j].c) < SMALL)
									{
										SET(In[n].a, In[j].c);
										In[j].index = -1;
										condense = true;
										break;
									} 
									else if(dist_Point_to_Line(In[n].c, In[n].b, In[j].c) < SMALL)
									{
										SET(In[n].c, In[j].c);
										In[j].index = -1;
										condense = true;
										break;
									}
									else if(Convex(In[n].a, In[n].b, In[n].c, In[j].c))
									{
										SET(In[n].d, In[j].c);
										In[j].index = -1;
										In[n].type = 4;
										condense = true;
										break;
									}
								}
								ROTDIR(In[n].a, In[n].b, In[n].c);
							}
							if(condense) break;
							ROTDIR(In[j].a, In[j].b, In[j].c);
							
						}
						if(condense) break;
					}
				} while(condense);
			} 
			In[i].index=-1;
		}
		NumInMax=NumInLast;
	
		// Quick and dirty fix to avoid creation of degenerated quads (with aligned vertices)
		// Note that the "flatness" calculated is not normalized, so the real value depends (as x^4!!!)
		// on actual size of the quad! - This is the way calculation is done in LDDP.
		#define DEGEN 0.0001
		degen=DEGEN*prescaling*prescaling*prescaling*prescaling;		

		for(i=0; i<NumInMax; i++)
		{
			if(In[i].type == 4)
			{
/*				SET(In[i].a, null);
				SET(In[i].b, null);
				SET(In[i].c, null);
				SET(In[i].d, null);
				
				In[i].a[0]=-1;
				In[i].b[0]=0;
				In[i].c[0]=1;
				In[i].d[1]=1;
*/				
				double a,b,c,d;
				a=flatness(In[i].d, In[i].a, In[i].b);
				b=flatness(In[i].a, In[i].b, In[i].c);
				c=flatness(In[i].b, In[i].c, In[i].d);
				d=flatness(In[i].c, In[i].d, In[i].a);
				if((a<degen)||(b<degen)||(c<degen)||(d<degen))
				{
					if((a<degen)||(c<degen))
					{
						if ((b>degen)&&(d>degen))
						{
							In[i].type = 3;
							LDRAWCPY(In[NumInMax], In[i]);
							SET(In[NumInMax].b,In[NumInMax].c);
							SET(In[NumInMax].c,In[NumInMax].d);
							NumInMax++;
						}
						continue;
					}
					else if ((b<degen)||(d<degen))
					{
						if ((a>degen)&&(c>degen))
						{
							In[i].type = 3;
							LDRAWCPY(In[NumInMax], In[i]);
							SET(In[i].c,In[i].d);
							SET(In[NumInMax].a,In[NumInMax].d);
							NumInMax++;
							continue;
						}
						continue;
					}
				}
			}
		}




		printf("Rebuilding uncut original quads...\n");

		for(i=0; i<NumInMax; i++)
		{
			int idx;
			int k,l, m;
			double t1[3][3], t2[3][3], q[4][3];
			int t1f[3],t2f[3];

			idx = In[i].index;
			if(idx < 0) continue;
			if(In[i].type !=3) continue;
			if(In[i].mod) continue;
			SET (t1[0],In[i].a);
			SET (t1[1],In[i].b);
			SET (t1[2],In[i].c);
			t1f[0]=t1f[1]=t1f[2]=true;
			t2f[0]=t2f[1]=t2f[2]=true;

			for(j=i+1; j<NumInMax; j++)
			{
				if(In[j].index != idx) continue;
				if(In[j].type !=3) continue;
				SET (t2[0],In[j].a);
				SET (t2[1],In[j].b);
				SET (t2[2],In[j].c);
				m=0;
				for (k=0;k<3;k++)
				{
					for(l=0;l<3;l++)
					{
						if(DIST(t1[k], t2[l]) < SMALL)
						{
							SET (q[2*m], t1[k]);
							t1f[k]=t2f[l]=false;
							m++;
						}
					}
				}
				if(m == 2)
				{
					for (k=0;k<3;k++)
					{
						if(t1f[k]) 
						{
							SET (q[1], t1[k]);
						}
						if(t2f[k]) 
						{
							SET (q[3], t2[k]);
						}
					}
					if(Convex(q[0], q[1], q[2], q[3]))
					{
						double a[3],b[3],v1[3],v2[3];
						// Maintain quad BFC direction identical to 1st triangle
						SUB(a, t1[0], t1[1]); 
						SUB(b, t1[0], t1[2]);
						CROSS(v1,a,b);
						SUB(a, q[0], q[1]); 
						SUB(b, q[0], q[2]);
						CROSS(v2,a,b);
						if(DOT(v1,v2)>0)
						{
							SET(In[i].a, q[0]);
							SET(In[i].b, q[1]);
							SET(In[i].c, q[2]);
							SET(In[i].d, q[3]);

						}
						else
						{
							SET(In[i].a, q[3]);
							SET(In[i].b, q[2]);
							SET(In[i].c, q[1]);
							SET(In[i].d, q[0]);
						}
						In[i].type=4;
						In[j].index=-1;
					}
				}
			}
		}

	}

	// Revert Prescaling
	prescaling = 1./prescaling;
	for(i=0; i<NumInMax; i++)
	{
		MULT(In[i].a, In[i].a, prescaling);
		MULT(In[i].b, In[i].b, prescaling);
		MULT(In[i].c, In[i].c, prescaling);
		MULT(In[i].d, In[i].d, prescaling);
	}


	printf("Writing output file...\n");

	fprintf(LdrawOut, "0 %s cut by %s\n", argv[argc-3], argv[argc-2]);
	fprintf(LdrawOut, "0 Name: %s\n", argv[argc-1]);
	fprintf(LdrawOut, "0 Author: Philo's Intersector\n0\n");
//	fprintf(LdrawOut, "0 BFC CERTIFY CCW\n\n");

	for(i=0;i<6;i++) n[i]=0;
	for(i=0; i<NumInMax; i++)
	{
		int idx;
		idx = In[i].index;
		if(idx < 0) continue;
		for(j=i; j<NumInMax; j++)
		{
			if(In[j].index != idx) continue;
			ltype=In[j].type;
			n[ltype]++;
			In[j].index=-1;
			fprintf(LdrawOut, "%d %d %.7lg %.7lg %.7lg %.7lg %.7lg %.7lg", ltype, In[j].color, In[j].a[0], In[j].a[1], In[j].a[2], In[j].b[0], In[j].b[1], In[j].b[2]);
			switch (ltype)
			{
			case 2:
				fprintf(LdrawOut, "\n");
				break;
			case 3:
				fprintf(LdrawOut, " %.7lg %.7lg %.7lg\n", In[j].c[0], In[j].c[1], In[j].c[2]);
				break;
			case 4:
			case 5:
				fprintf(LdrawOut, " %.7lg %.7lg %.7lg %.7lg %.7lg %.7lg\n", In[j].c[0], In[j].c[1], In[j].c[2], In[j].d[0], In[j].d[1], In[j].d[2]);
				break;
			}
		}
	}

	printf("  %d Line(s) written\n", n[2]);
	printf("  %d Triangle(s) written\n", n[3]);
	printf("  %d Quad(s) written\n", n[4]);
	printf("  %d Conditional Line(s) written\n", n[5]);
	fprintf(LdrawOut, "0\n");

	fclose (LdrawIn);
	fclose (LdrawCutter);
	fclose (LdrawOut);
	printf("Press <Enter> to quit");
	getchar();
	return(0);
}


